const didParser = require('./didParser')
const didResolver = require('./didResolver')

const didPattern = /^(did)\:([a-zA-Z0-9]+)\:([a-zA-Z0-9]{42})$/

const filterPublicKey = (doc, fragment) => {
  if (!doc) throw new Error('Document is not provided')
  if (!doc.publicKey) return new Error('Public key array is not found')
  if (!fragment) throw new Error('Fragment is not provided')
  return doc.publicKey.find(pubKey => pubKey.id === `${doc.id}${fragment}`)
}

const filterDelegates = doc => {
  if (!doc) throw new Error('Document is not provided')
  if (!doc.publicKey) return new Error('Public key array is not found')
  return doc.publicKey.filter(x => x.id.includes('delegate'))
}

const filterOwnerAddress = (doc, isSEC = true) => {
  const prefix = isSEC ? 'sec:' : '' // TODO: remove after solving the issue
  if (!doc) throw new Error('Document is not provided')
  const publicKey = filterPublicKey(doc, '#owner')
  if (!publicKey) throw new Error('Public key is not found')
  if (!(publicKey.ethereumAddress || publicKey[`${prefix}ethereumAddress`])) throw new Error('Owner address is not found')
  return (publicKey.ethereumAddress || publicKey[`${prefix}ethereumAddress`])
}

const filterService = (doc, serviceType) => {
  if (!doc) throw new Error('Document is not provided')
  if (!serviceType) throw new Error('Service type is not provided')
  if (!doc.service) return undefined
  return doc.service.find(service => service.type === serviceType)
}

const filterAgentServiceUrl = doc => {
  if (!doc) throw new Error('Document is not provided')
  const service = filterService(doc, 'AgentService')
  if (!service) return undefined
  return service.serviceEndpoint || undefined
}

const resolve = async (resolver, did, isSEC = true) => {
  if (!didPattern.test(did)) throw new Error(`Invalid DID: ${did}`)
  const { method } = didParser(did)
  const doc = await didResolver(resolver || 'UniResolverSpherity')(did)
  const delegates = filterDelegates(doc)
  const ownerAddress = filterOwnerAddress(doc, isSEC)
  const agentServiceUrl = filterAgentServiceUrl(doc)
  return {
    doc,
    owner: `did:${method}:${ownerAddress}`,
    delegates,
    agentServiceUrl
  }
}

module.exports = {
  resolve,
  filterPublicKey,
  filterDelegates,
  filterOwnerAddress,
  filterService,
  filterAgentServiceUrl
}
